@@ -58,7 +58,7 @@ class cmd_update(Command):
                         help=_('ignore all soft-dependencies')),
             ])
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
         module_set = jhbuild.moduleset.load(config)
         module_list = module_set.get_module_list(args or config.modules,
@@ -94,7 +94,7 @@ class cmd_updateone(Command):
                         help=_('set a sticky date when checking out modules')),
             ])
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
         module_set = jhbuild.moduleset.load(config)
         try:
@@ -128,7 +128,7 @@ class cmd_cleanone(Command):
                         help=_('honour the makeclean setting in config file')),
             ])
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         if options.honour_config is False:
             config.makeclean = True
         module_set = jhbuild.moduleset.load(config)
@@ -144,10 +144,9 @@ class cmd_cleanone(Command):
             logging.info(
                     _('clean command called while makeclean is set to False, skipped.'))
             return 0
-        config.build_targets = ['clean']
 
         build = jhbuild.frontends.get_buildscript(config, module_list)
-        return build.build()
+        return build.build(phases=['clean'])
 
 register_command(cmd_cleanone)
 
@@ -265,7 +264,7 @@ class cmd_build(Command):
                         help=_('skip modules installed less than the given time ago')),
             ])
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
 
         if not config.quiet_mode:
@@ -338,7 +337,7 @@ class cmd_buildone(Command):
                         help=_('skip modules installed less than the given time ago')),
             ])
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
 
         if not config.quiet_mode:
@@ -375,7 +374,7 @@ class cmd_run(Command):
                         help=_('run command in checkout dir of the given module')),
             ])
 
-    def execute(self, config, args):
+    def execute(self, config, args, help=None):
         # Do a shallow check of the arguments list
         # so that '--' isn't always required when command has arguments, 
         # only if some of them look like they might be for us
@@ -388,7 +387,7 @@ class cmd_run(Command):
             raise FatalError(_("Unable to execute the command '%(command)s': %(err)s") % {
                     'command':args[0], 'err':str(exc)})
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         if options.in_builddir:
             module_set = jhbuild.moduleset.load(config)
             try:
@@ -441,7 +440,7 @@ class cmd_shell(Command):
     name = 'shell'
     usage_args = ''
 
-    def execute(self, config, args):
+    def execute(self, config, args, help=None):
         if "--help" in args:
             self.parse_args(args) # This doesn't return
         user_shell = os.environ.get('SHELL', '/bin/sh')
@@ -481,7 +480,7 @@ class cmd_list(Command):
                         help=_('list all modules, not only those that would be built')),
             ])
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         config.set_from_cmdline_options(options)
         module_set = jhbuild.moduleset.load(config)
         if options.list_all_modules:
@@ -528,7 +527,7 @@ class cmd_dot(Command):
                         help=_('group modules from metamodule together')),
             ])
 
-    def run(self, config, options, args):
+    def run(self, config, options, args, help=None):
         module_set = jhbuild.moduleset.load(config)
         if args:
             modules = args
