@@ -298,6 +298,7 @@ class Config:
             libdir = os.path.join(self.prefix, 'lib64')
         else:
             libdir = os.path.join(self.prefix, 'lib')
+        self.libdir = libdir
         addpath('LD_LIBRARY_PATH', libdir)
 
         # LDFLAGS and C_INCLUDE_PATH are required for autoconf configure
@@ -323,6 +324,7 @@ class Config:
 
         # MANPATH
         manpathdir = os.path.join(self.prefix, 'share', 'man')
+        addpath('MANPATH', '')
         addpath('MANPATH', manpathdir)
 
         # PKG_CONFIG_PATH
@@ -341,6 +343,17 @@ class Config:
         addpath('PKG_CONFIG_PATH', pkgconfigdatadir)
         addpath('PKG_CONFIG_PATH', pkgconfigdir)
 
+        # GI_TYPELIB_PATH
+        if not 'GI_TYPELIB_PATH' in os.environ:
+            if self.use_lib64:
+                full_name = '/usr/lib64/girepository-1.0'
+            else:
+                full_name = '/usr/lib/girepository-1.0'
+            if os.path.exists(full_name):
+                addpath('GI_TYPELIB_PATH', full_name)
+        typelibpath = os.path.join(self.libdir, 'girepository-1.0')
+        addpath('GI_TYPELIB_PATH', typelibpath)
+
         # XDG_DATA_DIRS
         xdgdatadir = os.path.join(self.prefix, 'share')
         addpath('XDG_DATA_DIRS', xdgdatadir)
@@ -360,12 +373,21 @@ class Config:
                 os.makedirs(aclocaldir)
             except:
                 raise FatalError(_("Can't create %s directory") % aclocaldir)
+        if os.path.exists('/usr/share/aclocal'):
+            addpath('ACLOCAL_FLAGS', '/usr/share/aclocal')
+        if os.path.exists('/usr/local/share/aclocal'):
+            addpath('ACLOCAL_FLAGS', '/usr/local/share/aclocal')
         addpath('ACLOCAL_FLAGS', aclocaldir)
 
         # PERL5LIB
         perl5lib = os.path.join(self.prefix, 'lib', 'perl5')
         addpath('PERL5LIB', perl5lib)
 
+        # These two variables are so that people who use "jhbuild shell"
+        # can tweak their shell prompts and such to show "I'm under jhbuild".
+        # The first variable is the obvious one to look for; the second
+        # one is for historical reasons.
+        os.environ['UNDER_JHBUILD'] = 'true'
         os.environ['CERTIFIED_GNOMIE'] = 'yes'
 
         # PYTHONPATH
