@@ -35,6 +35,7 @@ from jhbuild import modtypes
 from jhbuild.versioncontrol import get_repo_type
 from jhbuild.utils import httpcache
 from jhbuild.utils.cmds import get_output
+from jhbuild.modtypes.testmodule import TestModule
 
 __all__ = ['load', 'load_tests']
 
@@ -283,7 +284,7 @@ def load(config, uri=None):
         modulesets = [ config.moduleset ]
     ms = ModuleSet(config = config)
     for uri in modulesets:
-        if '/' not in uri and not os.path.exists(uri):
+        if '/' not in uri and not os.path.isfile(uri):
             if config.modulesets_dir and config.nonetwork or config.use_local_modulesets:
                 uri = os.path.join(config.modulesets_dir, uri + '.modules')
             else:
@@ -298,7 +299,7 @@ def load_tests (config, uri=None):
     ms = load (config, uri)
     ms_tests = ModuleSet(config = config)
     for app, module in ms.modules.iteritems():
-        if module.__class__ == testmodule.TestModule:
+        if module.__class__ == TestModule:
             ms_tests.modules[app] = module
     return ms_tests
 
@@ -345,6 +346,8 @@ def _parse_module_set(config, uri):
             for attr in repo_class.init_xml_attrs:
                 if node.hasAttribute(attr):
                     kws[attr.replace('-', '_')] = node.getAttribute(attr)
+            if name in repositories:
+                logging.warning(_('Duplicate repository:') + ' '+ name)
             repositories[name] = repo_class(config, name, **kws)
             repositories[name].moduleset_uri = uri
             mirrors = {}
